Heka Docker Example
===================

This will guide you through a simple example combining Heka with Nginx, Kibana
and Elasticsearch, all running in each of their own containers, communicating with each other.

#### Notes

This assumes you're running Docker locally. If you're using Docker on a different
host, or in a VM, you should replace instances of `localhost` with the hostname
of the machine running Docker.

It also assumes you're in the `docker/example` directory where this `README` is
located.

Prerequisites
-------------

Make sure you're running a recent version of Docker. Docker `>= 1.0` should be fine,
but note I used docker version `1.1.0` with it.

### Download the required Docker images

````
docker pull arcus/kibana
docker pull dockerfile/elasticsearch
````

### Build the Heka image

````
cd ..
./build_docker.sh
````

Running the containers
----------------------

First we'll start up elasticsearch, and expose its HTTP and UDP ports.

````
docker run --name elasticsearch -d -p 9200:9200 -p 9300:9300 dockerfile/elasticsearch
````

You can check that its accessible by curling it like so:

````
$ curl localhost:9200
{
  "status" : 200,
  "name" : "Valkyrie",
  "version" : {
    "number" : "1.3.2",
    "build_hash" : "dee175dbe2f254f3f26992f5d7591939aaefd12f",
    "build_timestamp" : "2014-08-13T14:29:30Z",
    "build_snapshot" : false,
    "lucene_version" : "4.9"
  },
  "tagline" : "You Know, for Search"
}
````

Next we want to start up nginx with Kibana. In this command we're exposing port
80 on the host, so we can access Kibana from our web browser.

````
docker run --name kibana_nginx -d -p 80:80 -v /var/log/nginx arcus/kibana
````

You should be able to visit Kibana by going to `http://localhost/#/dashboard/file/guided.json`
in your web browser.

Finally we need to start Heka.

````
docker run --name heka -d -p 4352:4352 \
    --volumes-from kibana_nginx \
    --link elasticsearch:elasticsearch \
    -v `pwd`/example_config.toml:/etc/heka/example_config.toml \
    mozilla/heka -config /etc/heka/example_config.toml
````

Let's explain whats going on with the above command

In the first line we're exposing the port `4352` so we can view Heka's
dashboard. In the next line we mount the nginx volumes from our kibana
container, so Heka can process them. Finally we mount the config file in this
directory into the Heka container at `/etc/heka/example_config.toml` and then
run Heka with that config file.

You should now be able to refresh the Kibana page in your browser, or visit it at
`http://localhost/#/dashboard/file/guided.json` and after a few refreshes, Heka
will pick up on the log lines generated by visiting Kibana. These will be sent
by Heka to elasticsearch, which Kibana will then display for you.


Final Words
-----------

There are some places in this where we could have used more of Docker's features.
For example, we exposed Elasticsearch to the world, by publishing it's ports. A
better method would have been to use linking between Kibana and Elasticsearch,
but since Kibana does an elasticsearch lookup from the web browser client, this
is hard to do. Keep this in mind if you wish to try and modify the example with
Kibana.

It would also be beneficial to use a `data-only` container for nginx logs,
and then using `--volumes-from` to mount the container in both the `kibana_nginx`
container, as well as with the `heka` container..

We also never configured Elasticsearch, the defaults are fine for this example,
but you would want to adjust the settings before doing this in production.
